package me.unurled.sacredrealms.sr.utils;

import static me.unurled.sacredrealms.sr.utils.Component.comp;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.concurrent.atomic.AtomicReference;
import me.unurled.sacredrealms.sr.components.attributes.Attribute;
import me.unurled.sacredrealms.sr.components.item.Item;
import me.unurled.sacredrealms.sr.components.item.ItemManager;
import org.bukkit.Material;
import org.bukkit.entity.Player;
import org.bukkit.inventory.EntityEquipment;
import org.bukkit.inventory.ItemStack;
import org.bukkit.persistence.PersistentDataContainer;
import org.bukkit.persistence.PersistentDataType;
import org.jetbrains.annotations.NotNull;

public class Items {
  public static @NotNull Double getTotalAttribute(Player player, Attribute attribute) {
    return getTotalAttribute(player.getEquipment(), attribute);
  }

  public static @NotNull Double getTotalAttribute(EntityEquipment inv, Attribute attribute) {
    ItemManager im = (ItemManager) ItemManager.getInstance(ItemManager.class);
    if (im == null) return 0.0;

    AtomicReference<Double> total = new AtomicReference<>(0.0);

    Arrays.stream(inv.getArmorContents())
        .toList()
        .forEach(item -> total.updateAndGet(v -> (v + getAttribute(item, attribute))));

    total.updateAndGet(v -> (v + getAttribute(inv.getItemInMainHand(), attribute)));
    total.updateAndGet(v -> (v + getAttribute(inv.getItemInOffHand(), attribute)));

    return total.get();
  }

  public static Double getAttribute(ItemStack item, Attribute attribute) {
    ItemManager im = (ItemManager) ItemManager.getInstance(ItemManager.class);
    if (im == null) return 0.0;
    if (item == null) return 0.0;
    if (item.getType().equals(Material.AIR)) return 0.0;
    PersistentDataContainer p = item.getItemMeta().getPersistentDataContainer();
    if (p.has(ItemManager.ID, PersistentDataType.STRING)) {
      Item i = im.getItem(p.get(ItemManager.ID, PersistentDataType.STRING));
      if (i != null) {
        return i.getAttribute(attribute);
      }
    }
    return 0.0;
  }

  /** <b><color of the rarity>rarity
   * <dark_gray>Description: blablabla
   *   <dark_gray>blabbla
   *   <dark_gray>blabla
   * <gray>Attributes:
   *   <gray>➤ Health: <green>+100
   *   <gray>➤ Strength: <red>+10
   *   <gray>➤ Mana: <dark_aqua>+10
   * <gray>Abilities:
   *   <gray>➤ Ability 1,<yellow> On Rightclick unleash a powerful attack
   *               <dark_gray>cooldown: 10s,
   *               <dark_gray>damage: 100,
   *               <dark_gray>mana cost: <dark_aqua>50
   *   <gray>➤ Ability 2,<yellow> On Rightclick unleash a powerful attack
   *               <dark_gray>cooldown: 10s,
   *               <dark_gray>damage: 100,
   *               <dark_gray>mana cost: <dark_aqua>50
   * <dark_purple>Enchantments:
   *   <blue>➤ Enchantment <if max level gold>1
   *   <blue>➤ Enchantment <if max level gold>2
   *
   * @param item the item
   * @param stack the itemstack
   * @return the itemstack with lore
   * */
  public static ItemStack lore(ItemStack stack) {

    ItemManager im = (ItemManager) ItemManager.getInstance(ItemManager.class);
    if (im == null) return stack;
    if (stack == null) return stack;
    if (stack.getType().equals(Material.AIR)) return stack;
    PersistentDataContainer p = stack.getItemMeta().getPersistentDataContainer();
    if (!p.has(ItemManager.ID, PersistentDataType.STRING)) return stack;
    Item item = im.getItem(p.get(ItemManager.ID, PersistentDataType.STRING));

    List<net.kyori.adventure.text.Component> lore = new ArrayList<>();

    lore.add(comp("<b>" + item.getRarity().getName()));
    lore.add(comp("<dark_gray>" + item.getDescription()));
    if (!item.getAttributes().isEmpty()) {
      lore.add(comp("<gray>Attributes:"));
      item.getAttributes().forEach((k, v) -> lore.add(comp("  <gray>➤ " + k.getName() + ": " + (v < 0 ? "<red>" : "<green>") +
          "+" + v)));
    }
    if (!item.getAbilities().isEmpty())  {
      lore.add(comp("<gray>Abilities:"));
      item.getAbilities().forEach(a -> {
        lore.add(comp("  <gray>➤ " + a.getName() + ",<yellow> " + a.getDescription()));
        lore.add(comp("              <dark_gray>cooldown: " + a.getCooldown() + "s,"));
        lore.add(comp("              <dark_gray>damage: " + a.getDamage() + ","));
        lore.add(comp("              <dark_gray>mana cost: <dark_aqua>" + a.getManaCost()));
      });
    }
    if (!item.getEnchantments().isEmpty()) {
      lore.add(comp("<dark_purple>Enchantments:"));
      item.getEnchantments().forEach((k, v) -> lore.add(comp("  <blue>➤ " + k.getName() + " <if "
          + "max level gold>" + v)));
    }

    stack.lore(lore);

    return stack;
  }
}
